// ############################################################################################
//  Dct.cc
//  A demo application for VortexGE
//  -------------------------------------------------------------------------------------------
//  Copyright (C) 2004-2006 Aloysius Indrayanto
//  This file is part of VortexGE
//
//  This program is free software; you can redistribute it and/or modify it under the terms of
//  the GNU Lesser General Public License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public License along with this
//  library; if not, write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
//  Boston, MA 02111-1307 USA.
// ############################################################################################

// Common headers for demo application
#include "DemoCommon.h"


// Just for shortcut
typedef CBasicApplication<CEventHandler> MyApplicationBase;

// Specialized basic application class
class MyApplication : public MyApplicationBase {
  public:
    MyApplication();
    virtual void EventKeyPress(EKeySymbol eKeySym);
    virtual void EventUpdateFrame(bool bWindowIsActive);
    virtual void EventCloseWindow();

  private:
    CImageBuilder csIB1, csIB2;
    CImage        csIOrg, csIDCT, csIIDCT;
    uint8_t       un8Seq;
};


// Main program
int main()
{
  MyApplication csMyApp;
  return(csMyApp.DoMainLoop());
}


// MyApplication implementation
MyApplication::MyApplication()
: MyApplicationBase("Demo : Discrete Cosine Transform (DCT)", NULL, 640, 480, 0, 0, EAIM_Disabled),
  un8Seq(0)
{
  // Tell the user to be patient
  ShowInitSystemPleaseWaitMessage(iVideoEnv());

  // Load source image
  csIB1.LoadBMP("../media/image/back1.bmp");
  csIB1.ResizeToFitBackground();
  csIB1.SaveVIB("../.mediaCaches/dctdemo_rgb5-u.vib", EICM_None);
  csIB1.SaveVIB("../.mediaCaches/dctdemo_rgb5-c.vib", EICM_ZlibDeflate);
  //csIB1.LoadVIB("../.mediaCaches/dctdemo_rgb5-u.vib");
  csIB1.LoadVIB("../.mediaCaches/dctdemo_rgb5-c.vib");
  csIB1.TransferToImage(csIOrg);
  csIB2.CreateEmptyImage(csIB1);

  // Perform DCT
  //       Note that the DCT transformed image will become larger when saved
  csIB1.PerformDCT(csIB2, 4);
  csIB2.SaveVIB("../.mediaCaches/dctdemo_udct-u.vib", EICM_None);
  csIB2.SaveVIB("../.mediaCaches/dctdemo_udct-c.vib", EICM_ZlibDeflate);
  //csIB2.LoadVIB("../.mediaCaches/dctdemo_udct-u.vib");
  csIB2.LoadVIB("../.mediaCaches/dctdemo_udct-c.vib");
  csIB2.TransferCoefToImage(csIDCT);

  // Perform compaction & expansion
  // Note that the compacted DCT transformed image may still become
  // larger when saved, depends on the un8ReduceCoefficientStrength
  // parameter in the call to CImageBuilder::PerformDCT()
  csIB2.PerformDCTImageCompaction();
  csIB2.SaveVIB("../.mediaCaches/dctdemo_cdct-u.vib", EICM_None);
  csIB2.SaveVIB("../.mediaCaches/dctdemo_cdct-c.vib", EICM_ZlibDeflate);
  //csIB2.LoadVIB("../.mediaCaches/dctdemo_cdct-u");
  csIB2.LoadVIB("../.mediaCaches/dctdemo_cdct-c.vib");
  csIB2.PerformDCTImageExpansion();
  csIB2.TransferCoefToImage(csIDCT);

  // Perform inverse DCT
  csIB2.PerformInvDCT(csIB1, true);
  csIB1.BlurEffect(1);
  csIB1.SaveVIB("../.mediaCaches/dctdemo_rest-u.vib", EICM_None);
  csIB1.SaveVIB("../.mediaCaches/dctdemo_rest-c.vib", EICM_ZlibDeflate);
  //csIB1.LoadVIB("../.mediaCaches/dctdemo_rest-u.vib");
  csIB1.LoadVIB("../.mediaCaches/dctdemo_rest-c.vib");
  csIB1.TransferToImage(csIIDCT);

  // Display the file sizes
  VInfo(_NFL_, "Size File");
  system("ls -hs1 ../.mediaCaches/dctdemo*.vib");

  // Free images
  csIB1.FreeImage();
  csIB2.FreeImage();
}

void MyApplication::EventKeyPress(EKeySymbol eKeySym)
{
  switch(eKeySym) {
    case VKS_ESC : EventCloseWindow(); break;
    case ' '     : un8Seq = (un8Seq < 2) ? (un8Seq + 1) : 0; break;
    default      : break;
  }
}

void MyApplication::EventUpdateFrame(bool bWindowIsActive)
{
  // Update frame (if the application is not active)
  if(!bWindowIsActive) {
    iVideoEnv().PutRenderImageOnDisplay();
    iVideoEnv().SetForegroundColor(iVideoEnv().CreateColor(255, 255, 255));
    iVideoEnv().UseDefaultBigFont();
    iVideoEnv().DrawText(0, 150, "Inactive State", ETA_CenterScreen);
    iVideoEnv().UseDefaultSmallFont();
    iVideoEnv().UpdateDisplay();
    return;
  }

  // Display the background image
  switch(un8Seq) {
    case  0 : i3DEnv().ClearRenderBuffer(csIOrg); break;
    case  1 : i3DEnv().ClearRenderBuffer(csIDCT); break;
    case  2 : i3DEnv().ClearRenderBuffer(csIIDCT); break;
    default : break;
  }

  // Update display
  iVideoEnv().PutRenderImageOnDisplay();
  iVideoEnv().SetForegroundColor(iVideoEnv().CreateColor(255, 255, 255));
  iVideoEnv().DrawText(10, 20, "Press SPACE to display the next image");
  switch(un8Seq) {
    case  0 : iVideoEnv().DrawText(10, 468, "[Original Image]");      break;
    case  1 : iVideoEnv().DrawText(10, 468, "[Transformed Image]");   break;
    case  2 : iVideoEnv().DrawText(10, 468, "[Reconstructed Image]"); break;
    default : break;
  }
  iVideoEnv().DrawText(10, 50, "Press escape to exit ...");
  iVideoEnv().UpdateDisplay();
}

void MyApplication::EventCloseWindow()
{
  iVideoEnv().MessageBox("End Program", "Click \"OK\" to exit ...");
  MyApplicationBase::EventCloseWindow();
}
