// ############################################################################################
//  Dialog2.cc
//  A demo application for VortexGE
//  -------------------------------------------------------------------------------------------
//  Copyright (C) 2004-2006 Aloysius Indrayanto
//  This file is part of VortexGE
//
//  This program is free software; you can redistribute it and/or modify it under the terms of
//  the GNU Lesser General Public License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public License along with this
//  library; if not, write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
//  Boston, MA 02111-1307 USA.
// ############################################################################################

//#define __CONFIRM_TEXTBOX_EDIT_

// VortexGE headers
#include <VMain.h>


// Just for shortcut
typedef CDialogApplication<> MyApplicationBase;

// Foward reference of the application class
class MyApplication;


//
// Our dialog page class
//
class MyDialogPage : public CDialogPage {
  public:
    MyDialogPage(MyApplication &appClass);
    virtual ~MyDialogPage() throw();

    // Controls' event handlers
    void _onBtnExit(_CControl *ctrl);

    // Reimplemented from CDialogPage
    virtual void InitializeAllControls();
    virtual void HandleEvent_OnKeyPress(EKeySymbol keySymbol, const char *str);

  protected:
    // Reimplemented from CDialogPage
    virtual void DisplayTooltip(const char *tooltip);

    // Main data
    MyApplication &_app;
    CVideoStorage  _vStrg;
    // Controls
    CHLayout       hlyBase;
    CPanelBar      pbrMain;
    CHLayout       hlyMain;
    CMLTextBox     txtML;
    CButton        btnExit;
    CStatusBar     stbMain;

    // Callback functions
#ifdef __CONFIRM_TEXTBOX_EDIT_
#ifndef __USE_UTF8_DIALOG_STRING__
    static bool _mltbCallback(ETBEvent eventCode, CVideoEnv &iVideoEnv,
                              CMLTextBox &mlTextBox, vector<string> &editBuffer,
                              size_t actRow, size_t actCol,
                              vector<string> *pasteBuffer, char &charCode,
                              EKeySymbol keySym);
#else
    static bool _mltbCallback(ETBEvent eventCode, CVideoEnv &iVideoEnv,
                              CMLTextBox &mlTextBox, vector<CUString> &editBuffer,
                              size_t actRow, size_t actCol,
                              vector<CUString> *pasteBuffer, CUString::Utf32 &charCode,
                              EKeySymbol keySym);
#endif
#endif
};


//
// Our application class
//
class MyApplication : public MyApplicationBase {
  public:
    MyApplication();
    ~MyApplication() throw();

    // Standard event handlers
    void EventUpdateFrame(bool bWindowIsActive);
    void EventCloseWindow();

    // Make the dialog pages classes as friend classes
    friend class MyDialogPage;

  private:
    bool          _inactive;
    CVideoStorage _vStrg;
    MyDialogPage  _myDialogPage;
};


//
// Main program
//
int main()
{
  MyApplication csMyApp;
  return(csMyApp.DoMainLoop());
}


//
// MyDialogPage implementation
//
MyDialogPage::MyDialogPage(MyApplication &appClass)
: CDialogPage(appClass, &appClass),
  _app       (appClass)
{
  _vStrg.Initialize(_iVideoEnv, 1);
}

MyDialogPage::~MyDialogPage() throw()
{
}

void MyDialogPage::_onBtnExit(_CControl *ctrl)
{
  UNUSED_PARAM(ctrl);
  _app.EventCloseWindow();
}

void MyDialogPage::InitializeAllControls()
{
  InitControl(hlyBase, NULL, 0, 0);
      hlyBase.AddBlocksEx(2, 0, 25);
  InitControl(pbrMain, hlyBase, 0, "A Simple Dialog 2");
  InitControl(hlyMain, &pbrMain);
      hlyMain.AddBlocksEx(2, 0, 25);
#ifdef __CONFIRM_TEXTBOX_EDIT_
  InitControl(txtML, hlyMain, 0, true, _mltbCallback);
#else
  InitControl(txtML, hlyMain, 0);
#endif
      txtML.SetText("Line1 Line1 Line1 Line1 Line1\n\nLine2 Line2 Line2\nLine3\n\nLine4\nLine5\nLine6\nLine7\nLine8 Line8 Line8 Line8 Line8");
      //txtML.SetText("ぁあぃいぅうぇえぉお\n\n\nLine1 Line1 Line1 Line1 Line1\n\nLine2 Line2 Line2\nLine3\n\nLine4\nLine5\nLine6\nLine7\nLine8 Line8 Line8 Line8 Line8");
  InitControl(btnExit, hlyMain, 1, "Click to exit", true);
      Connect(btnExit, CDP_SLOT(MyDialogPage, _onBtnExit));
      btnExit.SetTooltip("Click to exit");
  InitControl(stbMain, hlyBase, 1, "");
}

void MyDialogPage::HandleEvent_OnKeyPress(EKeySymbol keySymbol, const char *str)
{
  UNUSED_PARAM(str);
  if(keySymbol == VKS_ESC) _app.EventCloseWindow();
}

void MyDialogPage::DisplayTooltip(const char *tooltip)
{
  stbMain.SetString(tooltip);
  stbMain.DrawControl();
}

#ifdef __CONFIRM_TEXTBOX_EDIT_
#ifndef __USE_UTF8_DIALOG_STRING__
bool MyDialogPage::_mltbCallback(ETBEvent eventCode, CVideoEnv &iVideoEnv,
                                 CMLTextBox &mlTextBox, vector<string> &editBuffer,
                                 size_t actRow, size_t actCol,
                                 vector<string> *pasteBuffer, char &charCode,
                                 EKeySymbol keySym)
#else
bool MyDialogPage::_mltbCallback(ETBEvent eventCode, CVideoEnv &iVideoEnv,
                                 CMLTextBox &mlTextBox, vector<CUString> &editBuffer,
                                 size_t actRow, size_t actCol,
                                 vector<CUString> *pasteBuffer, CUString::Utf32 &charCode,
                                 EKeySymbol keySym)
#endif
{
  UNUSED_PARAMS_7(mlTextBox, editBuffer, actRow, actCol, pasteBuffer, charCode, keySym);

  if(eventCode == ETBE_ConfirmOK) {
    if(iVideoEnv.MessageBox("Multi Line Text Box", "Accept change(s) ?", 
                            EMBI_Question, EMBB_YesNo) == EMBRV_No)
      return(false);
  }
  else if(eventCode == ETBE_ConfirmCancel) {
    if(iVideoEnv.MessageBox("Multi Line Text Box", "Cancel change(s) ?", 
                            EMBI_Question, EMBB_YesNo) == EMBRV_No)
      return(false);
  }
  return(true);
}
#endif


//
// MyApplication implementation
//
MyApplication::MyApplication()
: MyApplicationBase("Demo : Simple Dialog Demo 2", NULL, 640, 480, 0, 0, EAIM_Disabled, E3DIM_Disabled),
  _inactive        (false),
  _myDialogPage    (*this)
{
  // Initialize video storage
  _vStrg.Initialize(&iVideoEnv(), 1);

  // Register the pages (the 1st page will become the main page)
  _dialogPageManager.RegisterPage(_myDialogPage, "My Dialog Page");

  // Initialize dialog
  _myDialogPage.InitializeAllControls();

  // Draw all for the 1st time
  _myDialogPage.RedrawAllControls();
  iVideoEnv().UpdateDisplay();
}

MyApplication::~MyApplication() throw()
{
}

void MyApplication::EventUpdateFrame(bool bWindowIsActive)
{
  if(!bWindowIsActive) {
    if(!_inactive) {
      _inactive = true;
      _vStrg.StoreCurrentView();
    }
    iVideoEnv().SetForegroundColor(iVideoEnv().CreateColor(255, 255, 255));
    iVideoEnv().UseDefaultBigFont();
    iVideoEnv().DrawText(0, 150, "Inactive State", ETA_CenterScreen);
    iVideoEnv().UseDefaultSmallFont();
    iVideoEnv().UpdateDisplay();
  }
  else {
    if(_inactive) {
      _inactive = false;
      _vStrg.RestoreCurrentView();
      iVideoEnv().UpdateDisplay();
    }
    usleep(4000);
  }
}

void MyApplication::EventCloseWindow()
{
  iVideoEnv().ApplyBlackScreen();
  iVideoEnv().MessageBox("End Program", "Click \"OK\" to exit ...");
  MyApplicationBase::EventCloseWindow();
}
