// ############################################################################################
//  tut01b.cc
//  A tutorial application for VortexGE
//  -------------------------------------------------------------------------------------------
//  Copyright (C) 2004-2006 Aloysius Indrayanto
//  This file is part of VortexGE
//
//  This program is free software; you can redistribute it and/or modify it under the terms of
//  the GNU Lesser General Public License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public License along with this
//  library; if not, write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
//  Boston, MA 02111-1307 USA.
// ############################################################################################

// The VortexGE include file
#include <VMain.h>


// Objects
static CVideoEnv csVideoEnv;
static C3DEnv    cs3DEnv;
static CAudioEnv &csAudioEnv = CAudioEnv::AutoDetect();


// Main program
int main()
{
  // Local variables
  CFPSControl csFPSCtrl;        // For FPS control
  char        acBuff[PATH_MAX]; // Processing string buffer

  // Initialize video
  csVideoEnv.CreateMainWindow(640, 480,
                              "Tutorial 1b : Initializing the System with FPS Limiter");
  csVideoEnv.CreateRenderImage(640, 480);
  csVideoEnv.CreateRenderBuffer(640, 480);
  cs3DEnv.Initialize(&csVideoEnv);
  csVideoEnv.SetForegroundColor(csVideoEnv.CreateColor(255, 192, 255));

  // Initialize audio
  csAudioEnv.QueryDeviceName(acBuff, sizeof(acBuff));
  csAudioEnv.OpenDevice(acBuff);
  csAudioEnv.InitializeDevice();
  csAudioEnv.ActivateDevice();

  // Main loop
  csVideoEnv.AutoRepeatOff();
  csFPSCtrl.SaveStartTicks();
  while(1) {
    // Frame rate control
    if(csFPSCtrl.PumpEventAndDelay(csVideoEnv)) continue;
    if(csFPSCtrl.AdjustDelay())
      sprintf(acBuff, "%05.1f fps, %06d delays", csFPSCtrl.GetFPS(),
                                                 csFPSCtrl.GetDelayFactor());

    // Update everything (the application is active)
    if(csVideoEnv.IsActive()) {
      // Check event
      if(csVideoEnv.CheckEvent()) {
        // Keyboard event
        if(csVideoEnv.EventKeyPress())
          VPrint("KeyPress   : %05u (%05lu))\n", csVideoEnv.GetKeySym(),
                                                 csVideoEnv.GetRawKeyCode());
        else if(csVideoEnv.EventKeyRelease())
          VPrint("KeyRelease : %05u (%05lu)\n", csVideoEnv.GetKeySym(),
                                                csVideoEnv.GetRawKeyCode());
        // Mouse event
        else if(csVideoEnv.EventButtonPress())
          VPrint("BtnPress   : %d\n", csVideoEnv.GetButtonSym());
        else if(csVideoEnv.EventButtonRelease()) {
          VPrint("BtnRelease : %d\n", csVideoEnv.GetButtonSym());
          if(csVideoEnv.GetButtonSym() == VBS_MB1) break;
        }
        // Other event
        else if(csVideoEnv.EventCloseWindow())
          break;
      }

      // Don't update till the delay counter reach zero
      if(csFPSCtrl.GetDelayCounter()) continue;

      // Update display
      cs3DEnv.ClearRenderBuffer(V_RGB(12, 12, 16));
      cs3DEnv.ClearZBuffer();
      csVideoEnv.PutRenderImageOnDisplay();
      csVideoEnv.DrawText(10, 20, acBuff);
      csVideoEnv.DrawText(10, 50, "Click the left mouse button to exit ...");
      csVideoEnv.UpdateDisplay();

      // Update sound
      csAudioEnv.UpdateBuffer();
    }
    // Update everything (the application is inactive)
    else {
      while(csVideoEnv.IsInactive()) {
        // Update display (only of the application window is visible)
        if(csVideoEnv.IsVisible()) {
          csVideoEnv.PutRenderImageOnDisplay();
          csVideoEnv.UseDefaultBigFont();
          csVideoEnv.DrawText(0, 150, "Inactive State", ETA_CenterScreen);
          csVideoEnv.UseDefaultSmallFont();
          csVideoEnv.UpdateDisplay();
        }

        // Pump event
        csVideoEnv.DispatchEvent();
        usleep(100000);
      }
    }
  }
  csFPSCtrl.SaveEndTicks();

  // Deactivate audio
  csAudioEnv.DeactivateDevice();

  // Calculate & display the avegare frame rate
  sprintf(acBuff, "Average frame rate = %.2f fps\n\nClick \"OK\" to exit ...",
          csFPSCtrl.GetAvgFPS());
  csVideoEnv.MessageBox("End Program", acBuff);
  VInfo(_N_, _F_, _L_, "Program ended, average frame rate = %.2f FPS",
        csFPSCtrl.GetAvgFPS());

  // OK !
  return(0);
}
