// ############################################################################################
//  ImageClasses.cc
//  Implementation of CTexture, CAnimatedTexture,
//  CImage, and CAnimatedImage classes
//  -------------------------------------------------------------------------------------------
//  Copyright (C) 2004-2006 Aloysius Indrayanto
//  This file is part of VortexGE
//
//  This library is free software; you can redistribute it and/or modify it under the terms of
//  the GNU Lesser General Public License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//
//  This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public License along with this
//  library; if not, write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
//  Boston, MA 02111-1307 USA.
// ############################################################################################

// VortexGE headers
#include "../System/MemoryOperations.h"
#include "ImageClasses.h"


namespace vge {

//
// The texture class
//
CTexture::CTexture()
: _pun8Buffer(NULL), _un32Width    (0), _un32Height(0),
  _un32Mask  (0),    _un32MultShift(0)
{
  VCLASS_CAN_ABORT_ON_DESTRUCTION_CTOR_BEGIN("CTexture");
}

CTexture::~CTexture() throw()
{
  FreeTexture();
}

void CTexture::FreeTexture()
{
  if(_pun8Buffer) abmfree(_pun8Buffer);
  _bAligned      = false;
  _pun8Buffer    = NULL;
  _un32Width     = 0;
  _un32Height    = 0;
  _un32Mask      = 0;
  _un32MultShift = 0;
}


//
// The animated texture class
//
CAnimatedTexture::CAnimatedTexture()
: _pun8BufferArray(NULL), _pun8BufferMax(NULL), _un32FrameSize(0),
  _un32TotFrame   (0),    _un32CurFrame (0),    _un32StepDelay(0),
  _n8Step         (0)
{
  VCLASS_CAN_ABORT_ON_DESTRUCTION_CTOR_BEGIN("CAnimatedTexture");
}

CAnimatedTexture::~CAnimatedTexture() throw()
{
  FreeTexture();
}

void CAnimatedTexture::FreeTexture()
{
  if(_pun8BufferArray) abmfree(_pun8BufferArray);
  _pun8BufferArray = NULL;
  _pun8BufferMax   = NULL;
  _pun8Buffer      = NULL;
  _un32TotFrame    = 0;
  _un32CurFrame    = 0;
  _un32FrameSize   = 0;
  _un32StepDelay   = 0;
  _n8Step          = 0;
}


//
// The image class
//
CImage::CImage()
: _pun8Buffer(NULL), _un32Width(0), _un32Height(0), _un32SizeSL(0)
{
  VCLASS_CAN_ABORT_ON_DESTRUCTION_CTOR_BEGIN("CImage");
}

CImage::~CImage() throw()
{
  FreeImage();
}

void CImage::FreeImage()
{
  if(_pun8Buffer) abmfree(_pun8Buffer);
  _pun8Buffer = NULL;
  _un32Width  = 0;
  _un32Height = 0;
  _un32SizeSL = 0;
}


//
// The animated image class
//
CAnimatedImage::CAnimatedImage()
: _pun8BufferArray(NULL), _pun8BufferMax(NULL), _un32FrameSize(0),
  _un32TotFrame   (0),    _un32CurFrame (0),    _un32StepDelay(0),
  _n8Step         (0)
{
  VCLASS_CAN_ABORT_ON_DESTRUCTION_CTOR_BEGIN("CAnimatedImage");
}

CAnimatedImage::~CAnimatedImage() throw()
{
  FreeImage();
}

void CAnimatedImage::FreeImage()
{
  if(_pun8BufferArray) abmfree(_pun8BufferArray);
  _pun8BufferArray = NULL;
  _pun8BufferMax   = NULL;
  _pun8Buffer      = NULL;
  _un32TotFrame    = 0;
  _un32CurFrame    = 0;
  _un32FrameSize   = 0;
  _un32StepDelay   = 0;
  _n8Step          = 0;
}

} // namespace vge
