/** \file MTIMPlayer.h */
// ############################################################################################
//  MTIMPlayer.h
//  Definition & implementation of CMIDIPlayer class using libTiMidity
//  -------------------------------------------------------------------------------------------
//  Copyright (C) 2004-2006 Aloysius Indrayanto
//  This file is part of VortexGE
//
//  This library is free software; you can redistribute it and/or modify it under the terms of
//  the GNU Lesser General Public License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//
//  This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public License along with this
//  library; if not, write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
//  Boston, MA 02111-1307 USA.
// ############################################################################################
#include "../../VConfig.h"
#if !defined(_AUDIO_ENVIRONMENT___MID_LIBTIMIDITY___MTIM_PLAYER_H_) && defined(__USE_TIM_MIDI__)
#define _AUDIO_ENVIRONMENT___MID_LIBTIMIDITY___MTIM_PLAYER_H_

// VortexGE headers
#include "../MidiPlayer.h"


namespace vge {

  // Queue's constants
  const uint32_t WBUFF_QUEUES = uint32_t(32);
  const uint32_t WBUFF_QLIMIT = uint32_t(30);

  // Foward reference from "lib/AudioEnvironment/MID_libTiMidity/MTIMPlayer.cc"
  struct _TlibTiMidityData_;


  //!
  //! \brief libTiMidity MIDI player class
  //!
  //! Affecting environment variables :
  //!   - VGE_TIMIDITY_CFG_FILE
  //!
  //! See the section "Environment Variables" form the main doumentation (not the
  //! API documentation) for more details.
  //!
  //! \api pretty stable
  class VGE_EXPORT CTiMidityPlayer : public CMidiPlayer, VCLASS_SINGLETON {
    // Support for singleton class
    VCLASS_SINGLETON_DECLARE(CTiMidityPlayer)

    public:
      virtual ~CTiMidityPlayer() throw();
      // Initialization functions
      virtual void InitializeDevice(CAudioEnv *pcsVAudioEnv);
      // MIDI functions
      virtual void Play(bool bLooping = false);
      virtual bool IsPlaying() const { return(_bPlaying); }
      virtual void Stop();
      virtual void SetVolume(uint8_t un8VolProcent);
      // File I/O functions
      virtual void LoadMID(const CDataSource &source);

      // Friend classes
      friend class CMidiPlayer;

    protected:
      // Make sure user can't construct this class directly
      CTiMidityPlayer();

      // Mixing functions
      void _MixS8(unsigned char *pucDstSrcBuffer, const unsigned char *pucSrcBuffer, uint32_t un32Size);
      void _MixU8(unsigned char *pucDstSrcBuffer, const unsigned char *pucSrcBuffer, uint32_t un32Size);
      void _MixS16(unsigned char *pucDstSrcBuffer, const unsigned char *pucSrcBuffer, uint32_t un32Size);
      void _MixU16(unsigned char *pucDstSrcBuffer, const unsigned char *pucSrcBuffer, uint32_t un32Size);
      virtual void _MixWithBuffer(unsigned char *pucBuffer, uint32_t un32BuffSize, volatile bool *pbForceExit);
      // Clean-up functions
      void _CleanupSong(bool bExitMidi = false);
      // Thread functions
      static uint64_t _ThreadProc(TThreadData *ptThreadData);

      // Data
      uint32_t            _un32AChannels;       // Audio environment's primary buffer specifications
      uint32_t            _un32ABits;           // ---
      uint32_t            _un32ARates;          // ---
      uint32_t            _un32AFormat;         // ---
      uint32_t            _un32AFSize;          // ---
      _TlibTiMidityData_ *_ptD;                 // Pointer to libTiMidity data structure
      char               *_pcWQBuff;            // Queue buffer
      volatile bool       _bWBSt[WBUFF_QUEUES]; // Queue buffer cells' state
      uint32_t            _un32RPos;            // Queue buffer read position
      uint32_t            _un32WPos;            // Queue buffer write position
      volatile int32_t    _n32ToRead;           // Number of cells ready to be read (mixed)
      bool                _bLooping;            // Flags to indicate that the song is looped
      volatile bool       _bPlaying;            // Flags to indicate that the song is playing
      CThread             _csTConvert;          // The conversion thread
      CMutex              _csMutex;             // Synchronization mutex
      CAudioEnv          *_pcsAudEnv;           // Pointer to the corresponding audio environment
      bool                _blibTiMidityInit;    // Flags to indicate that the libTiMidity has been intialized

      // Pointer to mixing functions
      void (CTiMidityPlayer::*_pfnMFunc)(unsigned char *, const unsigned char *, uint32_t);
  };

} // namespace vge
#endif
