/**
 * \file WaveStructs.h
 * <hr>
 * \internal This file contains the constants and structures needed for working with
 *           Canonical WAV (Windows(TM) WAVE Audio) file format.
 * <hr>
 */
// ############################################################################################
//  WaveStructs.h
//  Definition of structures used by Canonical WAVE file
//  -------------------------------------------------------------------------------------------
//  Copyright (C) 2004-2006 Aloysius Indrayanto
//  This file is part of VortexGE
//
//  Thanks to Craig Stuart Sapp (craig[AT]ccrma.stanford.edu), for the Canonical WAVE format.
//
//  NOTE: The WAVE file format is a subset of Microsoft's RIFF specification for the storage
//        of multimedia files.
//
//  This library is free software; you can redistribute it and/or modify it under the terms of
//  the GNU Lesser General Public License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//
//  This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public License along with this
//  library; if not, write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
//  Boston, MA 02111-1307 USA.
// ############################################################################################
#ifndef _AUDIO_ENVIRONMENT___WAVE_STRUCTS_H_
#define _AUDIO_ENVIRONMENT___WAVE_STRUCTS_H_

// VortexGE headers
#include "../Base/VBasics.h"


namespace vge {

  //!
  //! \internal
  //! \brief Structures used by Windows(TM) Canonical WAVE file
  //!
  //! \api stable
  struct _attr_packed _TCWAVECHUNK {
    u_char   aucChunkID[4]; // Always contains string "RIFF"
    uint32_t un32ChunkSize; // 4 + (8 + un32SubChunk1Size) + (8 + un32SubChunk2Size)
    u_char   aucFormat[4];  // Always contains string "WAVE"
  };

  //!
  //! \internal
  //! \brief Structures used by Windows(TM) Canonical WAVE file
  //!
  //! \api stable
  struct _attr_packed _TFORMATSUBCHUNKPCM {
    u_char   aucSubChunk1ID[4]; // Always contains string "fmt "
    uint32_t un32SubChunk1Size; // For PCM always 16
    uint16_t un16AudioFormat;   // For PCM always 1
    uint16_t un16Channels;      // 1 = mono, 2 = stereo
    uint32_t un32SampleRate;    // The sampling frequency
    uint32_t un32ByteRate;      // un32SampleRate * un16Channels * (un16BitsPerSample / 8)
    uint16_t un16BlockAlign;    // un16Channels * (un16BitsPerSample / 8)
    uint16_t un16BitsPerSample; // 8 = 8 bits, 16 = 16 bits
  };

  //!
  //! \internal
  //! \brief Structures used by Windows(TM) Canonical WAVE file
  //!
  //! \api stable
  struct _attr_packed _TDATASUBCHUNK {
    u_char   aucSubChunk2ID[4]; // Always contains string "data"
    uint32_t un32SubChunk2Size; // <number of samples> * un16Channels * (un16BitsPerSample / 8)
  };

  //!
  //! \internal
  //! \brief Structures used by Windows(TM) Canonical WAVE file
  //!
  //! \api stable
  struct _attr_packed _TCWFPCMHEADER {
    _TCWAVECHUNK        tChunk;
    _TFORMATSUBCHUNKPCM tFormat;
    _TDATASUBCHUNK      tData;
  };

} // namespace vge
#endif
