// ############################################################################################
//  Entities.cc
//  Implementation of various game entity classes
//  -------------------------------------------------------------------------------------------
//  Copyright (C) 2004-2006 Aloysius Indrayanto
//  This file is part of VortexGE
//
//  This library is free software; you can redistribute it and/or modify it under the terms of
//  the GNU Lesser General Public License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//
//  This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public License along with this
//  library; if not, write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
//  Boston, MA 02111-1307 USA.
// ############################################################################################

// VortexGE headers
#include "../2DManipulation/AnimatedImageController.h"
#include "Entities.h"


namespace vge {

//
// TInit_BasicChar implementation
//
const TInit_BasicChar::CInit_AnimSet *&TInit_BasicChar::CreateEmptyAnimationSet = CGenttyBaseAImage::CreateEmptyAnimationSet;


//
// CBasicCharacter implementation
//
CBasicCharacter::CBasicCharacter(const TInit_BasicChar &init)
: VIC_CGenttyBaseXSpaceEBC(init),
  VIS_CGenttyBaseAImage   (init)
{
}

CBasicCharacter::~CBasicCharacter() throw()
{
}


//
// CBasicCharacterSprite2D implementation
//
CBasicCharacterSprite2D::CBasicCharacterSprite2D(const TInit_BasicCharSpr2D &init)
: VIC_CBasicCharacter(init),
  _saimage           (CAnimatedImage_MP()),
  _saictl            (NULL)
{
  // Check
  vassert(!init.spriteInitPos || _saimage.MediaInst(),
          "If 'TInit_BasicCharSpr2D::spriteInitPos' is not NULL,\n'TInit_BasicCharSpr2D::spriteAImage' must not contain empty image also");

  // Initialize the sprite
  ReinitializeSprite2D(init.spriteAImage, init.spriteASInit, init.spriteInitPos, init.spriteInitAdv);
}

CBasicCharacterSprite2D::~CBasicCharacterSprite2D() throw()
{
  delete _saictl;
}

void CBasicCharacterSprite2D::ReinitializeSprite2D(const CAnimatedImage_MP &spriteAImage,
                                                   const CInit_AnimSet     *spriteASInit,
                                                   const vector<TFloatXYZ> *spriteInitPos,
                                                   const TInit_Spr2DAdv    &spriteInitAdv)
{
  // Check
  vassert(!spriteInitPos || spriteAImage.MediaInst(),
          "If the 'spriteInitPos' parameter is not NULL,\nthe 'spriteAImage' parameter must not contain empty image also");

  // Copy data
  _saimage = spriteAImage;

  // Initialize the advanced frame control class
  if(spriteASInit) {
    delete _saictl;
    _saictl = new CAImgControl;

    for(size_t i = 0; i < spriteASInit->size(); ++i) {
      // Get the total frames count
      const CInit_AnimSetFrame &fi = (*spriteASInit)[i];
      if(!fi.size()) continue;
      // Add the frames
      const uint32_t idx = _saictl->AddAnimationSet(fi.size());
      for(size_t j = 0; j < fi.size(); ++j) _saictl->AddASFrame(idx, fi[j]);
    }
  }

  // Initialize the initial sprites' positions
  if(spriteInitPos) {
    vector<float> pos(spriteInitPos->size() * 3);
    for(size_t i = 0; i < spriteInitPos->size(); ++i) {
      pos[i*3 + 0] = (*spriteInitPos)[i].x;
      pos[i*3 + 1] = (*spriteInitPos)[i].y;
      pos[i*3 + 2] = (*spriteInitPos)[i].z;
    }

    _sprite.Initialize(&pos[0], spriteInitPos->size(), _saimage.MediaInst(),
                       spriteInitAdv.pivot, spriteInitAdv.excludedColorVRGB,
                       spriteInitAdv.ambientLight, spriteInitAdv.omniLight,
                       spriteInitAdv.collisionRadius);
  }
}

void CBasicCharacterSprite2D::ReinitializeSprite2D(const CAnimatedImage_MP &spriteAImage,
                                                   const CInit_AnimSet     *spriteASInit,
                                                   const TFloatXYZ         *spriteInitPos,
                                                   const TInit_Spr2DAdv    &spriteInitAdv)
{
  if(spriteInitPos) {
    vector<TFloatXYZ> ip;
    ip.push_back(*spriteInitPos);
    ReinitializeSprite2D(spriteAImage, spriteASInit, &ip, spriteInitAdv);
  }
  else {
    const vector<TFloatXYZ> *null = NULL;
    ReinitializeSprite2D(spriteAImage, spriteASInit, null, spriteInitAdv);
  }
}

} // namespace vge
