/** \file FatalSignalHandler.h */
// ############################################################################################
//  FatalSignalHandler.h
//  Definition of fatal signal handler functions
//  -------------------------------------------------------------------------------------------
//  Copyright (C) 2004-2006 Aloysius Indrayanto
//  This file is part of VortexGE
//
//  This library is free software; you can redistribute it and/or modify it under the terms of
//  the GNU Lesser General Public License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//
//  This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public License along with this
//  library; if not, write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
//  Boston, MA 02111-1307 USA.
// ############################################################################################
#ifndef _SYSTEM___FATAL_SIGNAL_HANDLER_H_
#define _SYSTEM___FATAL_SIGNAL_HANDLER_H_

// VortexGE headers
#include "../Base/VBasics.h"


namespace vge {

  // User callback function for fatal signal
  // NOTE: * Without this user callback, the application will be terminated
  //         after printing some messages.
  //       * In case of SIGINT and SIGTSTP, returning true will continue the
  //         execution of the application, for other signals the application
  //         will be terminated unconditionally.
  //       * Causing any other fatal signal in the user callback will directly
  //         terminate the application.
  typedef bool (*PFNVortexFatalFuncCallback)(void *);


  // Handler installer
  //!
  //! Install a user fatal signal handler.
  //! \n\n
  //! \sntc
  extern VGE_EXPORT void VInstallFSH(PFNVortexFatalFuncCallback pfnUserCallback = NULL, void *pvUserData = NULL);
  //!
  //! Uninstall the previously installed user fatal signal handler.
  //! \n\n
  //! \sntc
  extern VGE_EXPORT void VUninstallFSH();
  //!
  //! Enable or disable ^C.
  extern VGE_EXPORT void VEnableCtrlC(bool bEnabled);
  //!
  //! Enable or disable ^Z.
  extern VGE_EXPORT void VEnableCtrlZ(bool bEnabled);

  // Dump stack
  //!
  //! Dump the current call stack.
  extern VGE_EXPORT void VDumpStack();

} // namespace vge
#endif
