// ############################################################################################
//  Color.cc
//  Implementation of named color functions
//  -------------------------------------------------------------------------------------------
//  Copyright (C) 2004-2006 Aloysius Indrayanto
//  This file is part of VortexGE
//
//  This library is free software; you can redistribute it and/or modify it under the terms of
//  the GNU Lesser General Public License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//
//  This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public License along with this
//  library; if not, write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
//  Boston, MA 02111-1307 USA.
// ############################################################################################

// VortexGE headers
#include "Color.h"


namespace vge {

// Helper macro for the defining named color constants below
#define _VX_COLOR_(R, G, B) (uint32_t(R) << 19) | (uint32_t(G) << 11) | (uint32_t(B) << 3)

// Named color constants
const XColor15 VColor::AliceBlue            = _VX_COLOR_(29, 30, 31); // HTML color #F0F8FF
const XColor15 VColor::AntiqueWhite         = _VX_COLOR_(30, 29, 26); // HTML color #FAEBD7
const XColor15 VColor::Aqua                 = _VX_COLOR_( 0, 31, 31); // HTML color #00FFFF
const XColor15 VColor::Aquamarine           = _VX_COLOR_(15, 31, 26); // HTML color #7FFFD4
const XColor15 VColor::Azure                = _VX_COLOR_(29, 31, 31); // HTML color #F0FFFF
const XColor15 VColor::Beige                = _VX_COLOR_(30, 30, 27); // HTML color #F5F5DC
const XColor15 VColor::Bisque               = _VX_COLOR_(31, 28, 24); // HTML color #FFE4C4
const XColor15 VColor::Black                = _VX_COLOR_( 0,  0,  0); // HTML color #000000
const XColor15 VColor::BlanchedAlmond       = _VX_COLOR_(31, 29, 25); // HTML color #FFEBCD
const XColor15 VColor::Blue                 = _VX_COLOR_( 0,  0, 31); // HTML color #0000FF
const XColor15 VColor::BlueViolet           = _VX_COLOR_(17,  5, 27); // HTML color #8A2BE2
const XColor15 VColor::Brown                = _VX_COLOR_(20,  5,  5); // HTML color #A52A2A
const XColor15 VColor::BurlyWood            = _VX_COLOR_(27, 22, 16); // HTML color #DEB887
const XColor15 VColor::CadetBlue            = _VX_COLOR_(12, 19, 19); // HTML color #5F9EA0
const XColor15 VColor::Chartreuse           = _VX_COLOR_(15, 31,  0); // HTML color #7FFF00
const XColor15 VColor::Chocolate            = _VX_COLOR_(26, 13,  4); // HTML color #D2691E
const XColor15 VColor::Coral                = _VX_COLOR_(31, 15, 10); // HTML color #FF7F50
const XColor15 VColor::CornFlowerBlue       = _VX_COLOR_(12, 18, 29); // HTML color #6495ED
const XColor15 VColor::CornSilk             = _VX_COLOR_(31, 30, 27); // HTML color #FFF8DC
const XColor15 VColor::Crimson              = _VX_COLOR_(27,  2,  7); // HTML color #DC143C
const XColor15 VColor::Cyan                 = _VX_COLOR_( 0, 31, 31); // HTML color #00FFFF
const XColor15 VColor::DarkBlue             = _VX_COLOR_( 0,  0, 17); // HTML color #00008B
const XColor15 VColor::DarkCyan             = _VX_COLOR_( 0, 17, 17); // HTML color #008B8B
const XColor15 VColor::DarkGoldenrod        = _VX_COLOR_(22, 16,  1); // HTML color #B8860B
const XColor15 VColor::DarkGray             = _VX_COLOR_(21, 21, 21); // HTML color #A9A9A9
const XColor15 VColor::DarkGreen            = _VX_COLOR_( 0, 12,  0); // HTML color #006400
const XColor15 VColor::DarkKhaki            = _VX_COLOR_(23, 22, 13); // HTML color #BDB76B
const XColor15 VColor::DarkMagenta          = _VX_COLOR_(17,  0, 17); // HTML color #8B008B
const XColor15 VColor::DarkOliveGreen       = _VX_COLOR_(10, 13,  6); // HTML color #556B2F
const XColor15 VColor::DarkOrange           = _VX_COLOR_(31, 17,  0); // HTML color #FF8C00
const XColor15 VColor::DarkOrchid           = _VX_COLOR_(19,  6, 25); // HTML color #9932CC
const XColor15 VColor::DarkRed              = _VX_COLOR_(17,  0,  0); // HTML color #8B0000
const XColor15 VColor::DarkSalmon           = _VX_COLOR_(28, 18, 15); // HTML color #E9967A
const XColor15 VColor::DarkSeaGreen         = _VX_COLOR_(17, 23, 17); // HTML color #8FBC8F
const XColor15 VColor::DarkSlateBlue        = _VX_COLOR_( 9,  7, 17); // HTML color #483D8B
const XColor15 VColor::DarkSlateGray        = _VX_COLOR_( 6, 10, 10); // HTML color #2F4F4F
const XColor15 VColor::DarkTurquoise        = _VX_COLOR_( 0, 25, 25); // HTML color #00CED1
const XColor15 VColor::DarkViolet           = _VX_COLOR_(18,  0, 26); // HTML color #9400D3
const XColor15 VColor::DeepPink             = _VX_COLOR_(31,  2, 18); // HTML color #FF1493
const XColor15 VColor::DeepSkyBlue          = _VX_COLOR_( 0, 23, 31); // HTML color #00BFFF
const XColor15 VColor::DimGray              = _VX_COLOR_(13, 13, 13); // HTML color #696969
const XColor15 VColor::DodgerBlue           = _VX_COLOR_( 4, 18, 31); // HTML color #1E90FF
const XColor15 VColor::Firebrick            = _VX_COLOR_(22,  4,  4); // HTML color #B22222
const XColor15 VColor::FloralWhite          = _VX_COLOR_(31, 30, 29); // HTML color #FFFAF0
const XColor15 VColor::ForestGreen          = _VX_COLOR_( 4, 17,  4); // HTML color #228B22
const XColor15 VColor::Fuchsia              = _VX_COLOR_(31,  0, 31); // HTML color #FF00FF
const XColor15 VColor::Gainsboro            = _VX_COLOR_(27, 27, 27); // HTML color #DCDCDC
const XColor15 VColor::GhostWhite           = _VX_COLOR_(30, 30, 31); // HTML color #F8F8FF
const XColor15 VColor::Gold                 = _VX_COLOR_(31, 26,  0); // HTML color #FFD700
const XColor15 VColor::Goldenrod            = _VX_COLOR_(27, 20,  4); // HTML color #DAA520
const XColor15 VColor::Gray                 = _VX_COLOR_(16, 16, 16); // HTML color #808080
const XColor15 VColor::Green                = _VX_COLOR_( 0, 16,  0); // HTML color #008000
const XColor15 VColor::GreenYellow          = _VX_COLOR_(21, 31,  6); // HTML color #ADFF2F
const XColor15 VColor::Honeydew             = _VX_COLOR_(29, 31, 29); // HTML color #F0FFF0
const XColor15 VColor::HotPink              = _VX_COLOR_(31, 13, 22); // HTML color #FF69B4
const XColor15 VColor::IndianRed            = _VX_COLOR_(25, 11, 11); // HTML color #CD5C5C
const XColor15 VColor::Indigo               = _VX_COLOR_( 9,  0, 16); // HTML color #4B0082
const XColor15 VColor::Ivory                = _VX_COLOR_(31, 31, 29); // HTML color #FFFFF0
const XColor15 VColor::Khaki                = _VX_COLOR_(29, 28, 17); // HTML color #F0E68C
const XColor15 VColor::Lavender             = _VX_COLOR_(28, 28, 30); // HTML color #E6E6FA
const XColor15 VColor::LavenderBlush        = _VX_COLOR_(31, 29, 30); // HTML color #FFF0F5
const XColor15 VColor::LawnGreen            = _VX_COLOR_(15, 30,  0); // HTML color #7CFC00
const XColor15 VColor::LemonChiffon         = _VX_COLOR_(31, 30, 25); // HTML color #FFFACD
const XColor15 VColor::LightBlue            = _VX_COLOR_(21, 26, 28); // HTML color #ADD8E6
const XColor15 VColor::LightCoral           = _VX_COLOR_(29, 16, 16); // HTML color #F08080
const XColor15 VColor::LightCyan            = _VX_COLOR_(27, 31, 31); // HTML color #E0FFFF
const XColor15 VColor::LightGoldenrodYellow = _VX_COLOR_(30, 30, 26); // HTML color #FAFAD2
const XColor15 VColor::LightGreen           = _VX_COLOR_(18, 29, 18); // HTML color #90EE90
const XColor15 VColor::LightGray            = _VX_COLOR_(26, 26, 26); // HTML color #D3D3D3
const XColor15 VColor::LightPink            = _VX_COLOR_(31, 22, 23); // HTML color #FFB6C1
const XColor15 VColor::LightSalmon          = _VX_COLOR_(31, 19, 15); // HTML color #FFA07A
const XColor15 VColor::LightSeaGreen        = _VX_COLOR_( 4, 22, 21); // HTML color #20B2AA
const XColor15 VColor::LightSkyBlue         = _VX_COLOR_(16, 25, 30); // HTML color #87CEFA
const XColor15 VColor::LightSlateGray       = _VX_COLOR_(14, 17, 19); // HTML color #778899
const XColor15 VColor::LightSteelBlue       = _VX_COLOR_(21, 24, 27); // HTML color #B0C4DE
const XColor15 VColor::LightYellow          = _VX_COLOR_(31, 31, 27); // HTML color #FFFFE0
const XColor15 VColor::Lime                 = _VX_COLOR_( 0, 31,  0); // HTML color #00FF00
const XColor15 VColor::LimeGreen            = _VX_COLOR_( 6, 25,  6); // HTML color #32CD32
const XColor15 VColor::Linen                = _VX_COLOR_(30, 29, 28); // HTML color #FAF0E6
const XColor15 VColor::Magenta              = _VX_COLOR_(31,  0, 31); // HTML color #FF00FF
const XColor15 VColor::Maroon               = _VX_COLOR_(16,  0,  0); // HTML color #800000
const XColor15 VColor::MediumAquamarine     = _VX_COLOR_(12, 25, 21); // HTML color #66CDAA
const XColor15 VColor::MediumBlue           = _VX_COLOR_( 0,  0, 25); // HTML color #0000CD
const XColor15 VColor::MediumOrchid         = _VX_COLOR_(23, 10, 26); // HTML color #BA55D3
const XColor15 VColor::MediumPurple         = _VX_COLOR_(18, 14, 26); // HTML color #9370D8
const XColor15 VColor::MediumSeaGreen       = _VX_COLOR_( 7, 22, 14); // HTML color #3CB371
const XColor15 VColor::MediumSlateBlue      = _VX_COLOR_(15, 13, 29); // HTML color #7B68EE
const XColor15 VColor::MediumSpringGreen    = _VX_COLOR_( 0, 30, 19); // HTML color #00FA9A
const XColor15 VColor::MediumTurquoise      = _VX_COLOR_( 9, 25, 25); // HTML color #48D1CC
const XColor15 VColor::MediumVioletRed      = _VX_COLOR_(24,  3, 16); // HTML color #C71585
const XColor15 VColor::MidnightBlue         = _VX_COLOR_( 3,  3, 14); // HTML color #191970
const XColor15 VColor::MintCream            = _VX_COLOR_(30, 31, 30); // HTML color #F5FFFA
const XColor15 VColor::MistyRose            = _VX_COLOR_(31, 28, 27); // HTML color #FFE4E1
const XColor15 VColor::Moccasin             = _VX_COLOR_(31, 28, 22); // HTML color #FFE4B5
const XColor15 VColor::NavajoWhite          = _VX_COLOR_(31, 27, 21); // HTML color #FFDEAD
const XColor15 VColor::Navy                 = _VX_COLOR_( 0,  0, 16); // HTML color #000080
const XColor15 VColor::OldLace              = _VX_COLOR_(31, 30, 28); // HTML color #FDF5E6
const XColor15 VColor::Olive                = _VX_COLOR_(16, 16,  0); // HTML color #808000
const XColor15 VColor::OliveDrab            = _VX_COLOR_(13, 17,  4); // HTML color #688E23
const XColor15 VColor::Orange               = _VX_COLOR_(31, 20,  0); // HTML color #FFA500
const XColor15 VColor::OrangeRed            = _VX_COLOR_(31,  8,  0); // HTML color #FF4500
const XColor15 VColor::Orchid               = _VX_COLOR_(27, 14, 26); // HTML color #DA70D6
const XColor15 VColor::PaleGoldenrod        = _VX_COLOR_(29, 28, 21); // HTML color #EEE8AA
const XColor15 VColor::PaleGreen            = _VX_COLOR_(18, 31, 18); // HTML color #98FB98
const XColor15 VColor::PaleTurquoise        = _VX_COLOR_(21, 29, 29); // HTML color #AFEEEE
const XColor15 VColor::PaleVioletRed        = _VX_COLOR_(26, 14, 18); // HTML color #D87093
const XColor15 VColor::PapayaWhip           = _VX_COLOR_(31, 29, 26); // HTML color #FFEFD5
const XColor15 VColor::PeachPuff            = _VX_COLOR_(31, 27, 22); // HTML color #FFDAB9
const XColor15 VColor::Peru                 = _VX_COLOR_(25, 16,  8); // HTML color #CD853F
const XColor15 VColor::Pink                 = _VX_COLOR_(31, 23, 25); // HTML color #FFC0CB
const XColor15 VColor::Plum                 = _VX_COLOR_(27, 19, 27); // HTML color #DDA0DD
const XColor15 VColor::PowderBlue           = _VX_COLOR_(21, 27, 28); // HTML color #B0E0E6
const XColor15 VColor::Purple               = _VX_COLOR_(16,  0, 16); // HTML color #800080
const XColor15 VColor::Red                  = _VX_COLOR_(31,  0,  0); // HTML color #FF0000
const XColor15 VColor::RosyBrown            = _VX_COLOR_(23, 17, 17); // HTML color #BC8F8F
const XColor15 VColor::RoyalBlue            = _VX_COLOR_( 8, 13, 27); // HTML color #4169E1
const XColor15 VColor::SaddleBrown          = _VX_COLOR_(17,  8,  2); // HTML color #8B4513
const XColor15 VColor::Salmon               = _VX_COLOR_(30, 16, 14); // HTML color #FA8072
const XColor15 VColor::SandyBrown           = _VX_COLOR_(30, 20, 12); // HTML color #F4A460
const XColor15 VColor::SeaGreen             = _VX_COLOR_( 6, 17, 11); // HTML color #2E8B57
const XColor15 VColor::Seashell             = _VX_COLOR_(31, 29, 29); // HTML color #FFF5EE
const XColor15 VColor::Sienna               = _VX_COLOR_(19, 10,  5); // HTML color #A0522D
const XColor15 VColor::Silver               = _VX_COLOR_(23, 23, 23); // HTML color #C0C0C0
const XColor15 VColor::SkyBlue              = _VX_COLOR_(16, 25, 29); // HTML color #87CEEB
const XColor15 VColor::SlateBlue            = _VX_COLOR_(13, 11, 25); // HTML color #6A5ACD
const XColor15 VColor::SlateGray            = _VX_COLOR_(14, 16, 18); // HTML color #708090
const XColor15 VColor::Snow                 = _VX_COLOR_(31, 30, 30); // HTML color #FFFAFA
const XColor15 VColor::SpringGreen          = _VX_COLOR_( 0, 31, 15); // HTML color #00FF7F
const XColor15 VColor::SteelBlue            = _VX_COLOR_( 9, 16, 22); // HTML color #4682B4
const XColor15 VColor::Tan                  = _VX_COLOR_(26, 22, 17); // HTML color #D2B48C
const XColor15 VColor::Teal                 = _VX_COLOR_( 0, 16, 16); // HTML color #008080
const XColor15 VColor::Thistle              = _VX_COLOR_(26, 23, 26); // HTML color #D8BFD8
const XColor15 VColor::Tomato               = _VX_COLOR_(31, 12,  9); // HTML color #FF6347
const XColor15 VColor::Turquoise            = _VX_COLOR_( 8, 27, 25); // HTML color #40E0D0
const XColor15 VColor::Violet               = _VX_COLOR_(29, 16, 29); // HTML color #EE82EE
const XColor15 VColor::Wheat                = _VX_COLOR_(30, 27, 22); // HTML color #F5DEB3
const XColor15 VColor::White                = _VX_COLOR_(31, 31, 31); // HTML color #FFFFFF
const XColor15 VColor::WhiteSmoke           = _VX_COLOR_(30, 30, 30); // HTML color #F5F5F5
const XColor15 VColor::Yellow               = _VX_COLOR_(31, 31,  0); // HTML color #FFFF00
const XColor15 VColor::YellowGreen          = _VX_COLOR_(19, 25,  6); // HTML color #9ACD32


// Convert 24-bit RGB to BT.601 YUV
// * The nominal range for R, G and B are from 0 to 255
void BT601_RGB2YUV(uint8_t un8R, uint8_t un8G, uint8_t un8B, uint8_t &un8Y, uint8_t &un8U, uint8_t &un8V)
{
  un8Y = uint8_t((( 66L * un8R + 129L * un8G +  25L * un8B + 128L) >> 8) +  16L);
  un8U = uint8_t(((-38L * un8R -  74L * un8G + 112L * un8B + 128L) >> 8) + 128L);
  un8V = uint8_t(((112L * un8R -  94L * un8G -  18L * un8B + 128L) >> 8) + 128L);
}

// Convert BT.601 YUV to 24-bit RGB
// * The nominal range for Y is from 16 to 235
// * The nominal range for U and V are from 16 to 240
void BT601_YUV2RGB(uint8_t un8Y, uint8_t un8U, uint8_t un8V, uint8_t &un8R, uint8_t &un8G, uint8_t &un8B)
{
  int32_t n32C = 298L * (un8Y - 16L);
  int32_t n32D = un8U - 128L;
  int32_t n32E = un8V - 128L;
  int32_t n32R = ((n32C               + 409L * n32E + 128) >> 8);
  int32_t n32G = ((n32C - 100L * n32D - 208L * n32E + 128) >> 8);
  int32_t n32B = ((n32C + 516L * n32D               + 128) >> 8);

  un8R = (n32R >= 0) ? ((n32R <= 255) ? uint8_t(n32R) : 255) : 0;
  un8G = (n32G >= 0) ? ((n32G <= 255) ? uint8_t(n32G) : 255) : 0;
  un8B = (n32B >= 0) ? ((n32B <= 255) ? uint8_t(n32B) : 255) : 0;
}

// Convert V_RGB to V_YUV
// * The nominal range for R, G and B are from 0 to 31
void V_RGB2YUV(uint8_t un8R, uint8_t un8G, uint8_t un8B, uint8_t &un8Y, uint8_t &un8U, uint8_t &un8V)
{
  un8Y = uint8_t( ( 77L * un8R + 150L * un8G +  29L * un8B + 128L) >> 8);
  un8U = uint8_t(((-40L * un8R -  79L * un8G + 119L * un8B + 128L) >> 8) + 16L);
  un8V = uint8_t(((119L * un8R - 100L * un8G -  19L * un8B + 128L) >> 8) + 16L);
}

// Convert V_YUV to V_RGB
// * The nominal range for Y, U and V are from 0 to 31
void V_YUV2RGB(uint8_t un8Y, uint8_t un8U, uint8_t un8V, uint8_t &un8R, uint8_t &un8G, uint8_t &un8B)
{
  int32_t n32C = 256L * un8Y;
  int32_t n32D = un8U - 16L;
  int32_t n32E = un8V - 16L;
  int32_t n32R = ((n32C               + 385L * n32E + 128) >> 8);
  int32_t n32G = ((n32C -  94L * n32D - 196L * n32E + 128) >> 8);
  int32_t n32B = ((n32C + 486L * n32D               + 128) >> 8);

  un8R = (n32R >= 0) ? ((n32R <= 31) ? uint8_t(n32R) : 31) : 0;
  un8G = (n32G >= 0) ? ((n32G <= 31) ? uint8_t(n32G) : 31) : 0;
  un8B = (n32B >= 0) ? ((n32B <= 31) ? uint8_t(n32B) : 31) : 0;
}

} // namespace vge

