// ############################################################################################
//  mcatedit-MCatEdit-CtlEH.cc
//  Implementation of mcatedit's CMCatEdit and its dialog page classes
//  (controls' event handlers)
//  -------------------------------------------------------------------------------------------
//  Copyright (C) 2004-2006 Aloysius Indrayanto
//  This file is part of VortexGE
//
//  This program is free software; you can redistribute it and/or modify it under the terms of
//  the GNU Lesser General Public License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public License along with this
//  library; if not, write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
//  Boston, MA 02111-1307 USA.
// ############################################################################################

// mcatedit's headers
#include "mcatedit-MCatEdit.h"

// C++ headers
#include <sstream>


//
// Our dialog page: Directory Selection
//
void CPage_SelDir::_onLstSDir(_CControl *ctrl, bool dblClick)
{
  UNUSED_PARAM(ctrl);

  if(!dblClick) return;

  string sel = lstSDir.GetSelectedItem();
  if(sel == "INDEX.cat") {
    _onBtnOK(NULL);
  }
  else {
    lblSDirPath.SetString(_listDir(sel.c_str()).c_str());
    lblSDirPath.DrawControl();
    lstSDir.DrawControl();
  }
  _iVideoEnv->UpdateDisplay();
}

void CPage_SelDir::_onBtnOK(_CControl *ctrl)
{
  UNUSED_PARAM(ctrl);
  (_mPage.*_pfnTI)(lblSDirPath.GetString());
  _app.SelectPage(CMCatEdit::PAGE_MAIN);
}

void CPage_SelDir::_onBtnCancel(_CControl *ctrl)
{
  UNUSED_PARAM(ctrl);
  _app.SelectPage(CMCatEdit::PAGE_MAIN);
}




//
// Our dialog page: Locale Management
//
void CPage_MLoc::_onBtnNew(_CControl *ctrl)
{
  UNUSED_PARAM(ctrl);

  _vStrg.StoreCurrentView();
  const string &ans = _iVideoEnv->InputBox(_M("Insert the Locale Name"),
                                           "en_US",
                                           EMBS_Medium,
                                           ETBT_TextNormal,
                                           false,
                                           CPage_MLoc::_sltbCallback);
  if(ans == "") {
    _vStrg.RestoreCurrentView();
    return;
  }

  // Validate the locale name
  {
    bool   invalid = false;
    string lcname  = ans;
    const string::size_type np = string::npos;
    const string::size_type ps = lcname.find_first_of('_');
    const string::size_type pd = lcname.find_first_of('.');
    const string::size_type pa = lcname.find_first_of('@');
    // Is it in the form of something like "en_IE.iso885915@euro" ?
    if(ps!=np && pd!=np && pa!=np && ps<pd && pd<pa) {
      const string ln = lcname.substr(0, ps);
      const string cn = lcname.substr(ps+1, pd-ps-1);
      const string en = lcname.substr(pd+1, pa-pd-1);
      const string an = lcname.substr(pa+1, np);
      if(ln.length()!=2 || cn.length()!=2 || en.empty() || an.empty()) invalid = true;
    }
    // Is it in the form of something like "en_IE.iso885915" ?
    else if(ps!=np && pd!=np && pa==np && ps<pd) {
      const string ln = lcname.substr(0, ps);
      const string cn = lcname.substr(ps+1, pd-ps-1);
      const string en = lcname.substr(pd+1, pa-pd-1);
      if(ln.length()!=2 || cn.length()!=2 || en.empty()) invalid = true;
    }
    // Is it in the form of something like "en_IE@euro" ?
    else if(ps!=np && pd==np && pa!=np && ps<pa) {
      const string ln = lcname.substr(0, ps);
      const string cn = lcname.substr(ps+1, pa-ps-1);
      const string an = lcname.substr(pa+1, np);
      if(ln.length()!=2 || cn.length()!=2 || an.empty()) invalid = true;
    }
    // Is it in the form of something like "en_IE" ?
    else if(ps!=np && pd==np && pa==np) {
      const string ln = lcname.substr(0, ps);
      const string cn = lcname.substr(ps+1, np);
      if(ln.length()!=2 || cn.length()!=2) invalid = true;
    }
    // It should be in the form of something like "en"
    else {
      const string ln = lcname;
      if(ln.length()!=2) invalid = true;
    }
    if(invalid) {
      string tmp  = _M("The locale name:\n");
             tmp += ans;
             tmp += "\n";
             tmp += _M("looks like an invalid locale name.\n\n");
             tmp += _M("Some examples of valid locale names are:\n");
             tmp += "    en_IE.iso885915@euro\n";
             tmp += "    en_IE.iso885915\n";
             tmp += "    en_IE@euro\n";
             tmp += "    en_IE\n";
             tmp += "    en";
      _iVideoEnv->MessageBox(_M("New Locale"),
                             tmp.c_str(),
                             EMBI_Error,
                             EMBB_OK,
                             EMBS_Small);
      _vStrg.RestoreCurrentView();
      return;
    }
  }

  // Check if already exist and add it if not
  if(_app.pageMain._theCatalog.HasTranslationTable(ans)) {
    _iVideoEnv->MessageBox(_M("New Locale"),
                           _M("Locale with exactly the same name already exist."),
                           EMBI_Error,
                           EMBB_OK,
                           EMBS_Small);
    _vStrg.RestoreCurrentView();
  }
  else {
    _iVideoEnv->UseDefaultWaitCursor();
    _app.pageMain._theCatalog.AddNewTranslationTable(ans);
    lstLoc.AddItem(ans.c_str());
    stable_sort(lstLoc.GetAllItems().begin(), lstLoc.GetAllItems().end());
    lstLoc.SelectItem(ans.c_str());
    lstLoc.DrawControl();
    _iVideoEnv->UpdateDisplay();
    _iVideoEnv->UseDefaultArrowCursor();
    _app.pageMain.UpdateAllControlsEnableStates();
  }
}

void CPage_MLoc::_onBtnDel(_CControl *ctrl)
{
  UNUSED_PARAM(ctrl);

  const char *sel = lstLoc.GetSelectedItem();
  if(!sel) return;

  if(!_app.pageMain._theCatalog.IsEmptyTranslationTable(sel)) {
    _vStrg.StoreCurrentView();
    if(_iVideoEnv->MessageBox(_M("Delete Locale"),
                              _M("The locale to be deleted already has some translations.\n\nSure to delete ?"),
                              EMBI_Question,
                              EMBB_YesNo,
                              EMBS_Small) == EMBRV_No) {
      _vStrg.RestoreCurrentView();
      return;
    }
  }

  _iVideoEnv->UseDefaultWaitCursor();
  _app.pageMain._theCatalog.DeleteTranslationTable(sel);
  lstLoc.DelItem(sel);
  lstLoc.DrawControl();
  _iVideoEnv->UpdateDisplay();
  _iVideoEnv->UseDefaultArrowCursor();
  _app.pageMain.UpdateAllControlsEnableStates();
}

void CPage_MLoc::_onLstLoc(_CControl *ctrl, bool dblClick)
{
  UNUSED_PARAM(ctrl);

  if(!dblClick) return;

  const char *ci = lstLoc.GetSelectedItem();
  if(!ci) return;

  _TTranslations *ctt = _app.pageMain._theCatalog.GetTranslationTable(ci);
  if(!ctt) return;

  _vStrg.StoreCurrentView();
  const string &ans = _iVideoEnv->InputBoxML(_M("Additional Information for This Locale"),
                                             ctt->info,
                                             EMBS_Medium);
  _vStrg.RestoreCurrentView();
  if(ans != "") ctt->info = ans;
}

void CPage_MLoc::_onBtnDone(_CControl *ctrl)
{
  UNUSED_PARAM(ctrl);

  if(!btnLocDone.GetEnable()) return;
  const char *sel = lstLoc.GetSelectedItem();

  if(sel) {
    if(_app.pageMain._theCatalog.SelectTranslationTable(sel))
      _app.pageMain.UpdateAllControlsEnableStates();
  }

  _app.SelectPage(CMCatEdit::PAGE_MAIN);
}




//
// Our dialog page: Find in ID or MT
//
void CPage_Find::_onBtnSearch(_CControl *ctrl)
{
  UNUSED_PARAM(ctrl);

  // Prepare the search
  TSearchOption opt;
  if(rdoFindOptID.GetValue()) opt.findIn = TSearchOption::OriginalMessage;
  else                        opt.findIn = TSearchOption::TranslatedMessage;
       if(rdoFindOptSingle.GetValue()) opt.matchingRule = TSearchOption::MatchOneWord;
  else if(rdoFindOptAll.GetValue())    opt.matchingRule = TSearchOption::MatchAllWords;
  else                                 opt.matchingRule = TSearchOption::MatchExactSequence;
  opt.wordsToFind   = txtFindOpt.GetText();
  opt.caseSensitive = chkFindOptCS.GetValue();

  // Do the search
  const uint32_t r = _app.pageMain._theCatalog.Search_Start(opt);
  if(!r) {
    _iVideoEnv->MessageBox(_M("Find Word(s)"), _M("No match found!"),
                           EMBI_Information, EMBB_OK, EMBS_Tiny);
    SetParam(0, 0, "");
    RedrawAllControls();
    _iVideoEnv->UpdateDisplay();
  }
  else {
    char tmp[1024];
    snprintf(tmp, sizeof(tmp), "Found %d match(es).", r);
    _iVideoEnv->MessageBox(_M("Find Word(s)"), tmp,
                           EMBI_Information, EMBB_OK, EMBS_Tiny);
    _updateSearchDisplay();
  }
}

void CPage_Find::_onBtnFirst(_CControl *ctrl)
{
  UNUSED_PARAM(ctrl);

  if(!_app.pageMain._theCatalog.Search_SelectFirstMatch()) {
    _vStrg.StoreCurrentView();
    _iVideoEnv->MessageBox(_M("Find Word(s)"), _M("No more match!"),
                           EMBI_Information, EMBB_OK, EMBS_Tiny);
    _vStrg.RestoreCurrentView();
    return;
  }
  _updateSearchDisplay();
}

void CPage_Find::_onBtnPrev(_CControl *ctrl)
{
  UNUSED_PARAM(ctrl);

  if(!_app.pageMain._theCatalog.Search_SelectPrevMatch()) {
    _vStrg.StoreCurrentView();
    _iVideoEnv->MessageBox(_M("Find Word(s)"), _M("No more match!"),
                           EMBI_Information, EMBB_OK, EMBS_Tiny);
    _vStrg.RestoreCurrentView();
    return;
  }
  _updateSearchDisplay();
}

void CPage_Find::_onBtnNext(_CControl *ctrl)
{
  UNUSED_PARAM(ctrl);

  if(!_app.pageMain._theCatalog.Search_SelectNextMatch()) {
    _vStrg.StoreCurrentView();
    _iVideoEnv->MessageBox(_M("Find Word(s)"), _M("No more match!"),
                           EMBI_Information, EMBB_OK, EMBS_Tiny);
    _vStrg.RestoreCurrentView();
    return;
  }
  _updateSearchDisplay();
}

void CPage_Find::_onBtnLast(_CControl *ctrl)
{
  UNUSED_PARAM(ctrl);

  if(!_app.pageMain._theCatalog.Search_SelectLastMatch()) {
    _vStrg.StoreCurrentView();
    _iVideoEnv->MessageBox(_M("Find Word(s)"), _M("No more match!"),
                           EMBI_Information, EMBB_OK, EMBS_Tiny);
    _vStrg.RestoreCurrentView();
    return;
  }
  _updateSearchDisplay();
}

void CPage_Find::_onBtnDone(_CControl *ctrl)
{
  UNUSED_PARAM(ctrl);
  _app.pageMain.UpdateAllControlsEnableStates();
  _app.SelectPage(CMCatEdit::PAGE_MAIN);
}




//
// Our dialog page: Main Page
//
void CPage_Main::_onMnuFileNew(_CControl *ctrl)
{
  UNUSED_PARAM(ctrl);
  _app.pageSDir.SetParam(&CPage_Main::_onMnuFileNew, _M("Directory in which the new catalog will be created:"));
  _app.SelectPage(CMCatEdit::PAGE_SDIR);
}

void CPage_Main::_onMnuFileNew(const string &dirName)
{
  const string &pt = dirName;
  const string  fl = pt + "INDEX.cat";

  if(VFileIsReadable(fl)) {
    string tmp  = _M("The selected directory:\n\"");
           tmp += pt;
           tmp += _M("\"\n\nalready contains a file named \"INDEX.cat\". Overwrite it ?");

    if(_iVideoEnv->MessageBox(_M("New Catalog"),
                              tmp.c_str(),
                              EMBI_Question,
                              EMBB_YesNo,
                              EMBS_Small) == EMBRV_No)
    return;
  }

  _app.SetWindowTitle(pt);

  _theCatalog.NewCatalog(pt);
  UpdateAllControlsEnableStates();
}

void CPage_Main::_onMnuFileOpen(_CControl *ctrl)
{
  UNUSED_PARAM(ctrl);
  _app.pageSDir.SetParam(&CPage_Main::_onMnuFileOpen, _M("Directory which contains the catalog to be loaded:"));
  _app.SelectPage(CMCatEdit::PAGE_SDIR);
}

void CPage_Main::_onMnuFileOpen(const string &dirName)
{
  const string &pt = dirName;
  const string  fl = dirName + "INDEX.cat";

  if(!VFileIsReadable(fl)) {
    string tmp  = _M("The selected directory:\n\"");
           tmp += pt;
           tmp += _M("\"\n\ndoes not contain a file named \"INDEX.cat\".");

    _iVideoEnv->MessageBox(_M("Open Catalog"),
                           tmp.c_str(),
                           EMBI_Error,
                           EMBB_OK,
                           EMBS_Small);
     return;
  }

  try {
    _iVideoEnv->UseDefaultWaitCursor();
    _theCatalog.OpenCatalog(pt);
    _iVideoEnv->UseDefaultArrowCursor();

    _app.SetWindowTitle(pt);
  }
  catch(...) {
    string tmp  = _M("Couldn't load catalog from the selected directory:\n\"");
           tmp += pt;
           tmp += "\"";

    _theCatalog.Clear();
    _iVideoEnv->UseDefaultArrowCursor();
    _iVideoEnv->MessageBox(_M("Open Catalog"),
                           tmp.c_str(),
                           EMBI_Error,
                           EMBB_OK,
                           EMBS_Small);
  }

  vector<string> ll;
  _theCatalog.ListAllLocales(ll);
  _app.pageMLoc.lstLoc.DelAllItems();
  _app.pageMLoc.lstLoc.AddItems(ll);
  stable_sort(_app.pageMLoc.lstLoc.GetAllItems().begin(), _app.pageMLoc.lstLoc.GetAllItems().end());
  UpdateAllControlsEnableStates();
}

void CPage_Main::_onMnuFileSave(_CControl *ctrl)
{
  UNUSED_PARAM(ctrl);

  _saveCurValues();

  try {
    _iVideoEnv->UseDefaultWaitCursor();
    _theCatalog.SaveCatalog();
    _iVideoEnv->UseDefaultArrowCursor();
  }
  catch(...) {
    string tmp  = _M("Couldn't save catalog to directory:\n\"");
           tmp += _theCatalog.CurrentPath();
           tmp += "\"";

    _iVideoEnv->UseDefaultArrowCursor();
    _vStrg.StoreCurrentView();
    _iVideoEnv->MessageBox(_M("Save Catalog"),
                           tmp.c_str(),
                           EMBI_Error,
                           EMBB_OK,
                           EMBS_Small);
    _vStrg.RestoreCurrentView();
  }
}

void CPage_Main::_onMnuFileSaveAs(_CControl *ctrl)
{
  UNUSED_PARAM(ctrl);
  _app.pageSDir.SetParam(&CPage_Main::_onMnuFileSaveAs, _M("Directory in which the catalog will be saved:"));
  _app.SelectPage(CMCatEdit::PAGE_SDIR);
}

void CPage_Main::_onMnuFileSaveAs(const string &dirName)
{
  _saveCurValues();

  const string &pt = dirName;
  const string  fl = pt + "INDEX.cat";

  if(VFileIsReadable(fl)) {
    string tmp  = _M("The selected directory:\n\"");
           tmp += pt;
           tmp += _M("\"\n\nalready contains a file named \"INDEX.cat\". Overwrite it ?");

    if(_iVideoEnv->MessageBox(_M("Save Catalog As"),
                              tmp.c_str(),
                              EMBI_Question,
                              EMBB_YesNo,
                              EMBS_Small) == EMBRV_No)
    return;
  }

  try {
    _iVideoEnv->UseDefaultWaitCursor();
    _theCatalog.SaveCatalog(pt);
    _iVideoEnv->UseDefaultArrowCursor();

    _app.SetWindowTitle(pt);
  }
  catch(...) {
    string tmp  = _M("Couldn't save catalog to directory\n\"");
           tmp += pt;
           tmp += "\"";

    _iVideoEnv->UseDefaultArrowCursor();
    _iVideoEnv->MessageBox(_M("Save Catalog As"),
                           tmp.c_str(),
                           EMBI_Error,
                           EMBB_OK,
                           EMBS_Small);
  }
}

void CPage_Main::_onMnuFileIndexInfo(_CControl *ctrl)
{
  UNUSED_PARAM(ctrl);

  DisplayTooltip("");
  _vStrg.StoreCurrentView();
  const string &ans = _iVideoEnv->InputBoxML(_M("Additional Information for INDEX.cat"),
                                             _theCatalog.KeyInfo(),
                                             EMBS_Medium);
  _vStrg.RestoreCurrentView();
  if(ans != "") _theCatalog.KeyInfo() = ans;
}

void CPage_Main::_onMnuFileLocale(_CControl *ctrl)
{
  UNUSED_PARAM(ctrl);

  _saveCurValues();
  _app.SelectPage(CMCatEdit::PAGE_MLOC);
}

void CPage_Main::_onMnuFileAbout(_CControl *ctrl)
{
  UNUSED_PARAM(ctrl);

  stringstream ss;
  ss << _app.WINDOW_TITLE << " " << CMCatEdit::TEXT_VERSION     << endl;
  ss << _M("Copyright (C)") << " 2004-2006 Aloysius Indrayanto" << endl << endl << endl;
  ss << _M("This program is part of VortexGE.")                 << endl;
  ss << _M("This program is licensed under the GNU LGPL.");

  DisplayTooltip("");
  _vStrg.StoreCurrentView();
  _iVideoEnv->MessageBox(_M("About \"VortexGE Message Catalog Editor\""), ss.str());
  _vStrg.RestoreCurrentView();
}

void CPage_Main::_onBtnIDAdd(_CControl *ctrl)
{
  UNUSED_PARAM(ctrl);

  _saveCurValues();
  _theCatalog.CreateNewKey();
  UpdateAllControlsEnableStates();
}

void CPage_Main::_onBtnIDDel(_CControl *ctrl)
{
  UNUSED_PARAM(ctrl);

  if(_theCatalog.HasNoKey()) return;

  if(_theCatalog.CurrentKeyHasTranslations()) {
    _vStrg.StoreCurrentView();
    EMsgBoxRetVal r = _iVideoEnv->MessageBox(_M("Delete ID/Key"),
                                             _M("The ID/key to be deleted already has some translations.\n\nSure to delete ?"),
                                             EMBI_Question,
                                             EMBB_YesNo,
                                             EMBS_Small);
    _vStrg.RestoreCurrentView();
    if(r == EMBRV_No) return;
  }

  _saveCurValues();
  _iVideoEnv->UseDefaultWaitCursor();
  _theCatalog.DeleteCurrentKey();
  _iVideoEnv->UseDefaultArrowCursor();
  UpdateAllControlsEnableStates();
}

void CPage_Main::_onBtnIDFirst(_CControl *ctrl)
{
  UNUSED_PARAM(ctrl);

  _saveCurValues();
  if(_theCatalog.SelectFirstKey()) UpdateAllControlsEnableStates();
}

void CPage_Main::_onBtnIDPrev(_CControl *ctrl)
{
  UNUSED_PARAM(ctrl);

  _saveCurValues();
  if(_theCatalog.SelectPreviousKey()) UpdateAllControlsEnableStates();
}

void CPage_Main::_onBtnIDNext(_CControl *ctrl)
{
  UNUSED_PARAM(ctrl);

  _saveCurValues();
  if(_theCatalog.SelectNextKey()) UpdateAllControlsEnableStates();
}

void CPage_Main::_onBtnIDLast(_CControl *ctrl)
{
  UNUSED_PARAM(ctrl);

  _saveCurValues();
  if(_theCatalog.SelectLastKey()) UpdateAllControlsEnableStates();
}

void CPage_Main::_onBtnIDFind(_CControl *ctrl)
{
  UNUSED_PARAM(ctrl);

  _saveCurValues();
  _app.pageFind.SetParam(true);
  _app.SelectPage(CMCatEdit::PAGE_FIND);
}

void CPage_Main::_onBtnMTFirst(_CControl *ctrl)
{
  UNUSED_PARAM(ctrl);

  _saveCurValues();
  _iVideoEnv->UseDefaultWaitCursor();
  bool ret = _theCatalog.SelectFirstUntranslatedMessage();
  _iVideoEnv->UseDefaultArrowCursor();
  if(ret)
    UpdateAllControlsEnableStates();
  else {
    _vStrg.StoreCurrentView();
    _iVideoEnv->MessageBox(_M("Search for Untranslated Message"),
                           _M("No more untranslated message before the current message"),
                           EMBI_Error,
                           EMBB_OK,
                           EMBS_Small);
    _vStrg.RestoreCurrentView();
  }
}

void CPage_Main::_onBtnMTPrev(_CControl *ctrl)
{
  UNUSED_PARAM(ctrl);

  _saveCurValues();
  _iVideoEnv->UseDefaultWaitCursor();
  bool ret = _theCatalog.SelectPreviousUntranslatedMessage();
  _iVideoEnv->UseDefaultArrowCursor();
  if(ret)
    UpdateAllControlsEnableStates();
  else {
    _vStrg.StoreCurrentView();
    _iVideoEnv->MessageBox(_M("Search for Untranslated Message"),
                           _M("No more untranslated message before the current message"),
                           EMBI_Error,
                           EMBB_OK,
                           EMBS_Small);
    _vStrg.RestoreCurrentView();
  }
}

void CPage_Main::_onBtnMTNext(_CControl *ctrl)
{
  UNUSED_PARAM(ctrl);

  _saveCurValues();
  _iVideoEnv->UseDefaultWaitCursor();
  bool ret = _theCatalog.SelectNextUntranslatedMessage();
  _iVideoEnv->UseDefaultArrowCursor();
  if(ret)
    UpdateAllControlsEnableStates();
  else {
    _vStrg.StoreCurrentView();
    _iVideoEnv->MessageBox(_M("Search for Untranslated Message"),
                           _M("No more untranslated message after the current message"),
                           EMBI_Error,
                           EMBB_OK,
                           EMBS_Small);
    _vStrg.RestoreCurrentView();
  }
}

void CPage_Main::_onBtnMTLast(_CControl *ctrl)
{
  UNUSED_PARAM(ctrl);

  _saveCurValues();
  _iVideoEnv->UseDefaultWaitCursor();
  bool ret = _theCatalog.SelectLastUntranslatedMessage();
  _iVideoEnv->UseDefaultArrowCursor();
  if(ret)
    UpdateAllControlsEnableStates();
  else {
    _vStrg.StoreCurrentView();
    _iVideoEnv->MessageBox(_M("Search for Untranslated Message"),
                           _M("No more untranslated message after the current message"),
                           EMBI_Error,
                           EMBB_OK,
                           EMBS_Small);
    _vStrg.RestoreCurrentView();
  }
}

void CPage_Main::_onBtnMTFind(_CControl *ctrl)
{
  UNUSED_PARAM(ctrl);

  _saveCurValues();
  _app.pageFind.SetParam(false);
  _app.SelectPage(CMCatEdit::PAGE_FIND);
}
