// ############################################################################################
//  roomedit-RoomEdit.h
//  Definition of roomedit's CRoomEdit class
//  -------------------------------------------------------------------------------------------
//  Copyright (C) 2004-2006 Aloysius Indrayanto
//  This file is part of VortexGE
//
//  This program is free software; you can redistribute it and/or modify it under the terms of
//  the GNU Lesser General Public License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public License along with this
//  library; if not, write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
//  Boston, MA 02111-1307 USA.
// ############################################################################################
#ifndef _UTIL___ROOMEDIT_ROOMEDIT_H_
#define _UTIL___ROOMEDIT_ROOMEDIT_H_

// roomedit's headers
#include "roomedit-DlgPages-Main.h"
#include "roomedit-DlgPages-QRP.h"
#include "roomedit-DlgPages-GZS.h"
#include "roomedit-DlgPages-PNS.h"


// Just for shortcut
typedef CDialogApplication<> CRoomEditBase;

//
// Our application class
//
class CRoomEdit : public CRoomEditBase {
  public:
    enum Mode { ModeNormal, ModeNew, ModeOpen };

    // Constants for versioning
    static const string TEXT_VERSION;
    static const string TEXT_YEAR;

    // Constants for page name
    static const string PAGE_MAIN; // Main page
    static const string PAGE_HELP; // Help page
    static const string PAGE_GSZ;  // Grid size selection page
    static const string PAGE_NPS;  // New settings plane page

  public:
    CRoomEdit(uint32_t wWidth, uint32_t wHeight, bool lowRes3D, Mode mode, const string &file);
    virtual ~CRoomEdit() throw();

    // Management functions
    void SetWindowTitle(const string &addText);
    void SelectPage(const string &pageName);

    // Standard event handlers
    virtual void EventKeyPress(EKeySymbol eKeySym, const char *str);
    virtual void EventUpdateFrame(bool bWindowIsActive);
    virtual void EventWindowActivated();
    virtual void EventWindowDeactivated();
    virtual void EventCloseWindow();

    // Make the main dialog page class as friend class
    friend class CPage_Main;

  private:
    // Constants for video storage
    static const uint32_t APP_INACTIVE_STORE = 0;
    static const uint32_t PRE_DIALOG_STORE   = 1;
    static const uint32_t VSTORAGE_CAPACITY  = 2;

    // Common data
    string        WINDOW_TITLE;
    EKeySymbol    eKey;
    CVideoStorage csVStrg;
    // Page data
    CPage_Main    pageMain;
    CPage_QRP     pageHelp;
    CPage_GSZ     pageGSZ;
    CPage_PNS     pageNPS;
};

#endif
